#!/usr/bin/python
'''
author: Tobias Weis

This script is invoked by blender.
It will
    - parse the input arguments
    - setup the scene, render layers and file output nodes
    - create the scene (add camera, lighting, create objects)
    - render the scene and save the files
'''

import bpy
from bpy import context, data, ops
import bpy_extras
from mathutils import Matrix, Vector
import numpy
import os
import sys
import argparse
import numpy as np



# set up the scene
def create_scene(num_cubes):

    # ------ Camera
    bpy.ops.object.camera_add()
    # set active
    cam = bpy.data.objects['Camera']
    bpy.context.scene.camera = cam
    cam.location[0] = 0.0
    cam.location[1] = -50.0
    cam.location[2] = 15

    cam.rotation_euler[0] = np.pi/2.5
    cam.rotation_euler[1] = 0.0
    cam.rotation_euler[2] = 0.0

    # ------ Light
    bpy.ops.object.lamp_add(type="SUN")
    #bpy.data.objects["Lamp"].energy = 0.2
    bpy.data.lamps["Sun"].energy = 0.2

    # ------ General Scene
    # ground plane
    bpy.ops.mesh.primitive_plane_add()
    plane = bpy.context.selected_objects[0]
    plane.dimensions = [1000,1000,0.01]
    plane.location = [0,0,0]

    # ------ Cubes
    for i in range(num_cubes):
        bpy.ops.mesh.primitive_cube_add()
        cube = bpy.context.selected_objects[-1]
        cube.location = np.random.randint(-30,30,3)
        cube.location[2] = np.random.randint(0,3)
        cube.rotation_euler[2] = np.random.random(1) * np.pi


def setup(outp):
    # ------ Scene and Context
    bpy.data.scenes.new('MyScene')

    # set scene as active
    bpy.data.screens['Default'].scene = bpy.data.scenes["MyScene"]
    bpy.context.screen.scene = bpy.data.scenes["MyScene"]

    # setup render layers
    bpy.context.scene.render.layers["RenderLayer"].use_pass_combined = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_z = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_ambient_occlusion = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_diffuse_direct = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_diffuse_indirect = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_diffuse_color = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_glossy_direct = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_glossy_indirect = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_glossy_color = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_transmission_direct = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_transmission_indirect = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_transmission_color = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_subsurface_direct = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_subsurface_indirect = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_subsurface_color = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_emit = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_environment = True
    bpy.context.scene.render.layers["RenderLayer"].use_pass_normal = True

    # setup node tree and outputs
    bpy.data.scenes["MyScene"].use_nodes = True
    nodes = bpy.data.scenes["MyScene"].node_tree.nodes
    render_layers = nodes['Render Layers']

    output_file = nodes.new("CompositorNodeOutputFile")
    #output_file.inputs.new(type="RGBA", name="Depth")
    output_file.layer_slots.new("Depth")
    output_file.file_slots["Depth"].use_node_format = False
    output_file.file_slots["Depth"].format.file_format = "OPEN_EXR"
    print(dir(output_file.file_slots["Depth"]))
    #output_file.layer_slots[1].file_format = 'OPEN_EXR_MULTILAYER'
    #output_file.output_file_add_socket()
    #output_file.outputs[1].name = "Depth"
    output_file.base_path = outp

    # link image
    bpy.data.scenes["MyScene"].node_tree.links.new(
            render_layers.outputs['Image'],
            output_file.inputs['Image']
            )

    # link depth
    bpy.data.scenes["MyScene"].node_tree.links.new(
            render_layers.outputs['Depth'],
            output_file.inputs['Depth']
            )



if __name__ == "__main__":
    # parse variables given by the parent script
    parser = argparse.ArgumentParser(description="Render images and optical flow and save cameras")
    parser.add_argument('--outpath', type=str)
    parser.add_argument('--num_cubes', type=int)

    argv=sys.argv
    argv=argv[argv.index("--") + 1:]
    args=parser.parse_args(argv)

    if args.outpath is not None:
        outp = args.outpath
    else:
        outp = "/tmp/output/"

    if args.num_cubes is not None:
        num_cubes = args.num_cubes
    else:
        num_cubes = 50

    # create scene
    setup(outp)
    create_scene(num_cubes = 500) # fixme


    #bpy.context.scene.node_tree.nodes["File Output"].base_path = outp
    bpy.ops.render.render(True)
